@@ -41,7 +41,10 @@ from robocorp_ls_core.debug_adapter_core.dap.dap_schema import (
     OutputEventBody,
 )
 
-from robotframework_ls.impl.robot_constants import BUILTIN_VARIABLES
+from robotframework_ls.impl.robot_constants import (
+    get_builtin_variables,
+    ROBOT_AND_TXT_FILE_EXTENSIONS,
+)
 
 
 @lru_cache(None)
@@ -49,7 +52,7 @@ def get_builtin_normalized_names() -> FrozenSet[str]:
     from robotframework_ls.impl.text_utilities import normalize_robot_name
 
     normalized = list()
-    for k, _ in BUILTIN_VARIABLES:
+    for k, _ in get_builtin_variables():
         normalized.append(normalize_robot_name(k))
     return frozenset(normalized)
 
@@ -169,9 +172,9 @@ class _NonBuiltinVariablesAsDAP(_BaseObjectToDAP):
         return lst
 
     def _accept(self, k: str) -> bool:
-        from robotframework_ls.impl.text_utilities import normalize_robot_name
+        from robotframework_ls.impl.variable_resolve import normalize_variable_name
 
-        if normalize_robot_name(k) in self._builtins:
+        if normalize_variable_name(k) in self._builtins:
             return False
         else:
             return True
@@ -967,7 +970,7 @@ class _RobotDebuggerImpl(object):
 
         if not source:
             return
-        if not source.endswith((".robot", ".resource", ".txt")):
+        if not source.endswith(ROBOT_AND_TXT_FILE_EXTENSIONS):
             robot_init = os.path.join(source, "__init__.robot")
             if os.path.exists(robot_init):
                 source = robot_init
