@@ -18,6 +18,7 @@ class DebugListener(object):
     on_start_test = _Callback()
     on_end_test = _Callback()
     on_log_message = _Callback()
+    on_message = _Callback()
 
     def start_suite(self, data, result):
         self.on_start_suite(data, result)
@@ -34,6 +35,9 @@ class DebugListener(object):
     def log_message(self, message):
         self.on_log_message(message)
 
+    def message(self, message):
+        self.on_message(message)
+
     # def start_keyword(self, data, result):
     #     # This would be nice, but it's not currently supported.
     #
