@@ -1,4 +1,5 @@
 from typing import Dict, Optional
+import re
 
 from robocorp_ls_core.protocols import check_implements
 from robocorp_ls_core.robotframework_log import get_logger
@@ -9,8 +10,14 @@ from robotframework_ls.impl.protocols import (
     ICompletionContext,
     ILibraryDoc,
     INode,
+    IVariableFound,
 )
 from robocorp_ls_core.lsp import DiagnosticSeverity, DiagnosticTag
+from robotframework_ls.impl.robot_lsp_constants import (
+    OPTION_ROBOT_LINT_VARIABLES,
+    OPTION_ROBOT_LINT_IGNORE_VARIABLES,
+)
+from functools import lru_cache
 
 
 log = get_logger(__name__)
@@ -47,16 +54,40 @@ class _KeywordContainer(object):
         return None
 
 
+class _VariablesCollector(object):
+    def __init__(self, on_unresolved_variable_import):
+        self._variables_collected = set()
+        self.on_unresolved_variable_import = on_unresolved_variable_import
+
+    def accepts(self, variable_name: str) -> bool:
+        from robotframework_ls.impl.variable_resolve import normalize_variable_name
+
+        self._variables_collected.add(normalize_variable_name(variable_name))
+        # We don't want to create the IVariableFound, just the names should be
+        # enough for our usage.
+        return False
+
+    def on_variable(self, variable_found: IVariableFound):
+        pass
+
+    def contains_variable(self, variable_name):
+        return variable_name in self._variables_collected
+
+
 class _AnalysisKeywordsCollector(object):
     def __init__(
-        self, on_unresolved_library, on_unresolved_resource, on_resolved_library
+        self,
+        on_unresolved_library,
+        on_unresolved_resource,
+        on_resolved_library,
     ):
         self._keywords_container = _KeywordContainer()
         self._resource_name_to_keywords_container = {}
         self._library_name_to_keywords_container = {}
-        self._on_unresolved_library = on_unresolved_library
-        self._on_unresolved_resource = on_unresolved_resource
-        self._on_resolved_library = on_resolved_library
+
+        self.on_unresolved_library = on_unresolved_library
+        self.on_unresolved_resource = on_unresolved_resource
+        self.on_resolved_library = on_resolved_library
 
     def accepts(self, keyword_name):
         return True
@@ -124,56 +155,6 @@ class _AnalysisKeywordsCollector(object):
 
         return None
 
-    def on_resolved_library(
-        self,
-        completion_context: ICompletionContext,
-        library_node: Optional[INode],
-        library_doc: ILibraryDoc,
-    ):
-        self._on_resolved_library(
-            completion_context,
-            library_node,
-            library_doc,
-        )
-
-    def on_unresolved_library(
-        self,
-        completion_context: ICompletionContext,
-        library_name: str,
-        lineno: int,
-        end_lineno: int,
-        col_offset: int,
-        end_col_offset: int,
-        error_msg: Optional[str],
-    ):
-        self._on_unresolved_library(
-            completion_context,
-            library_name,
-            lineno,
-            end_lineno,
-            col_offset,
-            end_col_offset,
-            error_msg,
-        )
-
-    def on_unresolved_resource(
-        self,
-        completion_context: ICompletionContext,
-        resource_name: str,
-        lineno: int,
-        end_lineno: int,
-        col_offset: int,
-        end_col_offset: int,
-    ):
-        self._on_unresolved_resource(
-            completion_context,
-            resource_name,
-            lineno,
-            end_lineno,
-            col_offset,
-            end_col_offset,
-        )
-
     def __typecheckself__(self) -> None:
         _: IKeywordCollector = check_implements(self)
 
@@ -260,17 +241,11 @@ def collect_analysis_errors(initial_completion_context):
             if error_msg is None:
                 error_msg = ""
             else:
-                error_msg = f"\nError generating libspec:\n{error_msg}"
-
                 if "expected" in error_msg:
-                    import re
-
                     if re.search(r"expected\s+(.*)\s+argument(s)?,\s+got", error_msg):
                         additional = f'\nConsider using default arguments in the {library_name} constructor and\ncalling the "Robot Framework: Clear caches and restart" action or\nadding "{library_name}" to the "robot.libraries.libdoc.needsArgs"\nsetting to pass the typed arguments when generating the libspec.'
 
                 if not additional and "Importing" in error_msg:
-                    import re  # noqa
-
                     if get_robot_major_version() <= 3:
                         pattern = r"Importing\s+test\s+library(.*)failed"
                     else:
@@ -293,6 +268,7 @@ def collect_analysis_errors(initial_completion_context):
         end_lineno: int,
         col_offset: int,
         end_col_offset: int,
+        error_msg: Optional[str],
     ):
         from robotframework_ls.impl.robot_lsp_constants import (
             OPTION_ROBOT_LINT_UNDEFINED_RESOURCES,
@@ -307,12 +283,15 @@ def collect_analysis_errors(initial_completion_context):
         if doc and doc.uri == initial_completion_context.doc.uri:
             start = (lineno - 1, col_offset)
             end = (end_lineno - 1, end_col_offset)
-            errors.append(
-                ast_utils.Error(f"Unresolved resource: {library_name}", start, end)
-            )
+            if not error_msg:
+                error_msg = f"Unresolved resource: {library_name}"
+
+            errors.append(ast_utils.Error(error_msg, start, end))
 
     collector = _AnalysisKeywordsCollector(
-        on_unresolved_library, on_unresolved_resource, on_resolved_library
+        on_unresolved_library,
+        on_unresolved_resource,
+        on_resolved_library,
     )
     collect_keywords(initial_completion_context, collector)
 
@@ -364,14 +343,6 @@ def collect_analysis_errors(initial_completion_context):
                 )
 
                 keyword_token = None
-                if keyword_usage_info.is_argument_usage:
-                    try:
-                        keyword_token = keyword_usage_info.node.get_tokens(
-                            Token.ARGUMENT
-                        )[keyword_usage_info.argument_usage_index]
-                    except IndexError:
-                        pass
-
                 if keyword_token is None:
                     keyword_token = keyword_usage_info.node.get_token(Token.KEYWORD)
 
@@ -380,7 +351,6 @@ def collect_analysis_errors(initial_completion_context):
                         UsageInfoForKeywordArgumentAnalysis(
                             keyword_usage_info.node,
                             keyword_token,
-                            keyword_usage_info.argument_usage_index,
                         )
                     ):
                         errors.append(error)
@@ -401,4 +371,215 @@ def collect_analysis_errors(initial_completion_context):
         except:
             log.exception("Error collecting exceptions")
 
+    for error in _collect_undefined_variables_errors(initial_completion_context):
+        errors.append(error)
+        if len(errors) >= MAX_ERRORS:
+            # i.e.: Collect at most 100 errors
+            break
+
     return errors
+
+
+def _is_number_var(normalized_variable_name):
+    # see: robot.variables.finders.NumberFinder
+    try:
+        bases = {"0b": 2, "0o": 8, "0x": 16}
+        if normalized_variable_name.startswith(tuple(bases)):
+            return int(
+                normalized_variable_name[2:], bases[normalized_variable_name[:2]]
+            )
+        int(normalized_variable_name)
+        return True
+    except:
+        pass  # Let's try float...
+
+    try:
+        float(normalized_variable_name)
+        return True
+    except:
+        pass
+
+    return False
+
+
+def _is_python_eval_var(normalized_variable_name):
+    return (
+        len(normalized_variable_name) >= 2
+        and normalized_variable_name[0] == "{"
+        and normalized_variable_name[-1] == "}"
+    )
+
+
+@lru_cache(maxsize=1000)
+def _skip_variable_analysis(normalized_variable_name):
+
+    if _is_number_var(normalized_variable_name):
+        return True
+
+    if _is_python_eval_var(normalized_variable_name):
+        return True
+
+    return False
+
+
+_match_extended = re.compile(
+    r"""
+    (.+?)          # base name (group 1)
+    ([^\s\w].+)    # extended part (group 2)
+""",
+    re.UNICODE | re.VERBOSE,
+).match
+
+
+def _extract_base_from_extended_var_name(normalized_variable_name):
+    m = _match_extended(normalized_variable_name)
+    if m is None:
+        return normalized_variable_name
+
+    base_name, _extended = m.groups()
+    return base_name
+
+
+def _env_vars_upper():
+    import os
+
+    return tuple(x.upper() for x in os.environ)
+
+
+def _collect_undefined_variables_errors(initial_completion_context):
+    from robotframework_ls.impl import ast_utils
+    from robotframework_ls.impl.variable_resolve import normalize_variable_name
+    from robotframework_ls.impl.ast_utils import create_error_from_node
+
+    config = initial_completion_context.config
+    if config is not None and not config.get_setting(
+        OPTION_ROBOT_LINT_VARIABLES, bool, True
+    ):
+        return
+
+    unresolved_variable_import_errors = []
+
+    def on_unresolved_variable_import(
+        completion_context: ICompletionContext,
+        library_name: str,
+        lineno: int,
+        end_lineno: int,
+        col_offset: int,
+        end_col_offset: int,
+        error_msg: Optional[str],
+    ):
+        from robotframework_ls.impl.robot_lsp_constants import (
+            OPTION_ROBOT_LINT_UNDEFINED_VARIABLE_IMPORTS,
+        )
+
+        if config is not None and not config.get_setting(
+            OPTION_ROBOT_LINT_UNDEFINED_VARIABLE_IMPORTS, bool, True
+        ):
+            return
+
+        doc = completion_context.doc
+        if doc and doc.uri == initial_completion_context.doc.uri:
+            start = (lineno - 1, col_offset)
+            end = (end_lineno - 1, end_col_offset)
+            if not error_msg:
+                error_msg = f"Unresolved variable import: {library_name}"
+
+            unresolved_variable_import_errors.append(
+                ast_utils.Error(error_msg, start, end)
+            )
+
+    from robotframework_ls.impl import variable_completions
+
+    ignore_variables = set()
+    if config is not None:
+        ignore_variables.update(
+            normalize_variable_name(str(x))
+            for x in config.get_setting(OPTION_ROBOT_LINT_IGNORE_VARIABLES, list, [])
+        )
+
+    ast = initial_completion_context.get_ast()
+
+    globals_collector = _VariablesCollector(
+        on_unresolved_variable_import=on_unresolved_variable_import
+    )
+
+    # Collect undefined variables
+    variable_completions.collect_global_variables(
+        initial_completion_context, globals_collector, only_current_doc=False
+    )
+
+    yield from iter(unresolved_variable_import_errors)
+
+    env_vars_upper = None
+
+    for token_info in ast_utils.iter_variable_references(ast):
+        initial_completion_context.check_cancelled()
+
+        if token_info.node.__class__.__name__ in (
+            "ResourceImport",
+            "LibraryImport",
+            "VariableImport",
+        ):
+            # These ones are handled differently as it ends up in an unresolved
+            # import.
+            continue
+
+        var_name = token_info.token.value
+        if token_info.var_identifier == "%":
+            from robotframework_ls.impl.variable_resolve import extract_variable_base
+
+            if env_vars_upper is None:
+                env_vars_upper = _env_vars_upper()
+
+            if extract_variable_base(var_name).upper() not in env_vars_upper:
+                # Environment variable
+                yield create_error_from_node(
+                    token_info.node,
+                    f"Undefined environment variable: {token_info.token.value}",
+                    tokens=[token_info.token],
+                )
+            continue
+
+        normalized_variable_name = normalize_variable_name(var_name)
+
+        found = False
+        while not found:
+            if normalized_variable_name in ignore_variables:
+                found = True
+                break
+
+            if _skip_variable_analysis(normalized_variable_name):
+                found = True
+                break
+
+            if globals_collector.contains_variable(normalized_variable_name):
+                found = True
+                break
+
+            locals_collector = _VariablesCollector(lambda *args, **kwargs: None)
+            local_ctx = initial_completion_context.create_copy_with_selection(
+                line=token_info.token.lineno - 1,
+                col=token_info.token.col_offset - 1,
+            )
+
+            variable_completions.collect_local_variables(
+                local_ctx, locals_collector, token_info
+            )
+            if locals_collector.contains_variable(normalized_variable_name):
+                found = True
+                break
+
+            extracted_base = _extract_base_from_extended_var_name(
+                normalized_variable_name
+            )
+            if extracted_base and extracted_base != normalized_variable_name:
+                normalized_variable_name = extracted_base
+            else:
+                break
+
+        if not found:
+            yield create_error_from_node(
+                token_info.node,
+                f"Undefined variable: {token_info.token.value}",
+                tokens=[token_info.token],
+            )
