@@ -5,6 +5,7 @@ from robotframework_ls.impl.protocols import ICompletionContext, NodeInfo
 from robocorp_ls_core.robotframework_log import get_logger
 from robocorp_ls_core.protocols import ITestInfoTypedDict
 from robocorp_ls_core.lsp import CommandTypedDict
+from robotframework_ls.impl.robot_constants import ROBOT_FILE_EXTENSIONS
 
 log = get_logger(__name__)
 
@@ -292,12 +293,19 @@ def code_lens_resolve(
 
 def code_lens(completion_context: ICompletionContext) -> List[CodeLensTypedDict]:
     import os
+    from robotframework_ls.impl.robot_lsp_constants import OPTION_ROBOT_SHOW_CODE_LENSES
+
+    config = completion_context.config
+    if config is not None and not config.get_setting(
+        OPTION_ROBOT_SHOW_CODE_LENSES, bool, False
+    ):
+        return []
 
     path = completion_context.doc.path
     if not path or not os.path.exists(os.path.dirname(path)):
         # The cwd must exist.
         return []
-    if not path.endswith((".robot", ".resource")):
+    if not path.endswith(ROBOT_FILE_EXTENSIONS):
         return []
 
     code_lenses = code_lens_runs(completion_context)
