@@ -1,4 +1,3 @@
-from collections import namedtuple
 import os.path
 
 from robocorp_ls_core.cache import instance_cache
@@ -9,9 +8,10 @@ from robotframework_ls.impl.protocols import (
     ICompletionContext,
     IKeywordCollector,
     IKeywordArg,
-    ILibraryDoc,
+    LibraryDependencyInfo,
+    AbstractKeywordCollector,
 )
-from typing import Sequence, List, Dict, Optional
+from typing import Sequence, List, Dict, Optional, Iterator
 from robotframework_ls.impl.text_utilities import build_keyword_docs_with_signature
 from robocorp_ls_core.lsp import MarkupContentTypedDict, MarkupKind
 
@@ -279,7 +279,6 @@ def _collect_completions_from_ast(
 ):
     from robotframework_ls.impl import ast_utils
     from robocorp_ls_core.lsp import CompletionItemKind
-    from robotframework_ls.impl.robot_specbuilder import KeywordArg
 
     found = {}
 
@@ -287,9 +286,7 @@ def _collect_completions_from_ast(
         completion_context.check_cancelled()
         keyword_name = keyword.node.name
         if collector.accepts(keyword_name):
-            keyword_args = []
-            for arg in ast_utils.iter_keyword_arguments_as_str(keyword.node):
-                keyword_args.append(KeywordArg(arg))
+            keyword_args = list(ast_utils.iter_keyword_arguments_as_kwarg(keyword.node))
 
             found[keyword_name] = _KeywordFoundFromAst(
                 ast,
@@ -319,63 +316,28 @@ def _collect_current_doc_keywords(
     _collect_completions_from_ast(ast, completion_context, collector)
 
 
-_LibInfo = namedtuple("_LibInfo", "name, alias, builtin, args, node")
-
-
 def _collect_libraries_keywords(
-    completion_context: ICompletionContext, collector: IKeywordCollector
+    completion_context: ICompletionContext,
+    current_doc_uri: str,
+    library_infos: Iterator[LibraryDependencyInfo],
+    collector: IKeywordCollector,
 ):
-    """
-    :param CompletionContext completion_context:
-    """
     from robotframework_ls.impl.libspec_manager import LibspecManager
     from robotframework_ls.impl.protocols import ILibraryDocOrError
 
     # Get keywords from libraries
-    from robotframework_ls.impl.robot_constants import BUILTIN_LIB
     from robocorp_ls_core.lsp import CompletionItemKind
-    from robotframework_ls.impl import ast_utils
 
-    libraries = completion_context.get_imported_libraries()
-
-    # Note: using a dict(_LibInfo:bool) where only the keys are meaningful
-    # because we want to keep the order and sets aren't ordered.
-    library_infos = {}
-    for name, alias, args, node in (
-        (
-            library.name,
-            library.alias,
-            ast_utils.get_library_arguments_serialized(library),
-            library,
-        )
-        for library in libraries
-    ):
-        if name:
-            lib_info = _LibInfo(
-                completion_context.token_value_resolving_variables(name),
-                alias,
-                False,
-                args,
-                node,
-            )
-
-            library_infos[lib_info] = True
-
-    library_infos[_LibInfo(BUILTIN_LIB, None, True, None, None)] = True
     libspec_manager: LibspecManager = completion_context.workspace.libspec_manager
 
     for library_info in library_infos:
         completion_context.check_cancelled()
-        if not completion_context.memo.complete_for_library(
-            library_info.name, library_info.alias
-        ):
-            continue
 
         library_doc_or_error: ILibraryDocOrError = (
             libspec_manager.get_library_doc_or_error(
                 library_info.name,
                 create=True,
-                current_doc_uri=completion_context.doc.uri,
+                current_doc_uri=current_doc_uri,
                 builtin=library_info.builtin,
                 args=library_info.args,
             )
@@ -410,49 +372,111 @@ def _collect_libraries_keywords(
             from robot.api import Token
 
             error_msg = library_doc_or_error.error
-
             node = library_info.node
-            node_name_tok = node.get_token(Token.NAME)
-            if node_name_tok is not None:
-                collector.on_unresolved_library(
-                    completion_context,
-                    node.name,
-                    node_name_tok.lineno,
-                    node_name_tok.lineno,
-                    node_name_tok.col_offset,
-                    node_name_tok.end_col_offset,
-                    error_msg,
-                )
-            else:
-                collector.on_unresolved_library(
-                    completion_context,
-                    library_info.name,
-                    node.lineno,
-                    node.end_lineno,
-                    node.col_offset,
-                    node.end_col_offset,
-                    error_msg,
-                )
+            if node:
+                if error_msg:
+                    error_msg = f"\nError generating libspec:\n{error_msg}"
+
+                node_name_tok = node.get_token(Token.NAME)
+
+                if node_name_tok is not None:
+                    (
+                        _value,
+                        token_errors,
+                    ) = completion_context.token_value_and_unresolved_resolving_variables(
+                        node_name_tok
+                    )
+                    if token_errors:
+                        for token_error in token_errors:
+                            collector.on_unresolved_library(
+                                completion_context,
+                                node.name,
+                                token_error.lineno,
+                                token_error.lineno,
+                                token_error.col_offset,
+                                token_error.end_col_offset,
+                                f"\nUnable to statically resolve variable: {token_error.value}.\nPlease set the `{token_error.value[2:-1]}` value in `robot.variables`.",
+                            )
+                    else:
+                        collector.on_unresolved_library(
+                            completion_context,
+                            node.name,
+                            node_name_tok.lineno,
+                            node_name_tok.lineno,
+                            node_name_tok.col_offset,
+                            node_name_tok.end_col_offset,
+                            error_msg,
+                        )
+                else:
+                    collector.on_unresolved_library(
+                        completion_context,
+                        library_info.name,
+                        node.lineno,
+                        node.end_lineno,
+                        node.col_offset,
+                        node.end_col_offset,
+                        error_msg,
+                    )
 
 
-def _collect_resource_imports_keywords(
+def _collect_from_context(
     completion_context: ICompletionContext, collector: IKeywordCollector
 ):
+    dependency_graph = completion_context.collect_dependency_graph()
+
+    root_doc = dependency_graph.get_root_doc()
+    assert root_doc.uri == completion_context.doc.uri
+    _collect_current_doc_keywords(completion_context, collector)
+
+    completion_context.check_cancelled()
+    _collect_libraries_keywords(
+        completion_context,
+        completion_context.doc.uri,
+        dependency_graph.iter_libraries(completion_context.doc.uri),
+        collector,
+    )
+
+    for node, resource_doc in dependency_graph.iter_all_resource_imports_with_docs():
+        completion_context.check_cancelled()
 
-    for node, resource_doc in completion_context.get_resource_imports_as_docs():
         if resource_doc is None:
+            # Note that 'None' documents will only be given for the
+            # initial context (so, it's ok to use `completion_context`
+            # in this case).
             from robot.api import Token
 
             node_name_tok = node.get_token(Token.NAME)
             if node_name_tok is not None:
-                collector.on_unresolved_resource(
-                    completion_context,
-                    node.name,
-                    node_name_tok.lineno,
-                    node_name_tok.lineno,
-                    node_name_tok.col_offset,
-                    node_name_tok.end_col_offset,
+
+                (
+                    _value,
+                    token_errors,
+                ) = completion_context.token_value_and_unresolved_resolving_variables(
+                    node_name_tok
                 )
+
+                if token_errors:
+                    for token_error in token_errors:
+                        collector.on_unresolved_resource(
+                            completion_context,
+                            node.name,
+                            token_error.lineno,
+                            token_error.lineno,
+                            token_error.col_offset,
+                            token_error.end_col_offset,
+                            f"\nUnable to statically resolve variable: {token_error.value}.\nPlease set the `{token_error.value[2:-1]}` value in `robot.variables`.",
+                        )
+
+                else:
+                    collector.on_unresolved_resource(
+                        completion_context,
+                        node.name,
+                        node_name_tok.lineno,
+                        node_name_tok.lineno,
+                        node_name_tok.col_offset,
+                        node_name_tok.end_col_offset,
+                        None,
+                    )
             else:
                 collector.on_unresolved_resource(
                     completion_context,
@@ -461,27 +485,22 @@ def _collect_resource_imports_keywords(
                     node.end_lineno,
                     node.col_offset,
                     node.end_col_offset,
+                    None,
                 )
             continue
+        completion_context.check_cancelled()
         new_ctx = completion_context.create_copy(resource_doc)
-        _collect_following_imports(new_ctx, collector)
+
+        _collect_current_doc_keywords(new_ctx, collector)
+        _collect_libraries_keywords(
+            new_ctx,
+            resource_doc.uri,
+            dependency_graph.iter_libraries(resource_doc.uri),
+            collector,
+        )
 
 
-def _collect_following_imports(
-    completion_context: ICompletionContext, collector: IKeywordCollector
-):
-    completion_context.check_cancelled()
-    if completion_context.memo.follow_import(completion_context.doc.uri):
-        # i.e.: prevent collecting keywords for the same doc more than once.
-
-        _collect_current_doc_keywords(completion_context, collector)
-
-        _collect_resource_imports_keywords(completion_context, collector)
-
-        _collect_libraries_keywords(completion_context, collector)
-
-
-class _CollectKeywordNameToKeywordFound:
+class _CollectKeywordNameToKeywordFound(AbstractKeywordCollector):
     def __init__(self) -> None:
         self.keyword_name_to_keyword_found: Dict[str, List[IKeywordFound]] = {}
 
@@ -494,37 +513,6 @@ class _CollectKeywordNameToKeywordFound:
             self.keyword_name_to_keyword_found[keyword_found.keyword_name] = lst = []
         lst.append(keyword_found)
 
-    def on_resolved_library(
-        self,
-        completion_context: ICompletionContext,
-        library_node,
-        library_doc: ILibraryDoc,
-    ):
-        pass
-
-    def on_unresolved_library(
-        self,
-        completion_context: ICompletionContext,
-        library_name: str,
-        lineno: int,
-        end_lineno: int,
-        col_offset: int,
-        end_col_offset: int,
-        error_msg: Optional[str],
-    ):
-        pass
-
-    def on_unresolved_resource(
-        self,
-        completion_context: ICompletionContext,
-        resource_name: str,
-        lineno: int,
-        end_lineno: int,
-        col_offset: int,
-        end_col_offset: int,
-    ):
-        pass
-
     def __typecheckself__(self) -> None:
         _: IKeywordCollector = check_implements(self)
 
@@ -532,9 +520,8 @@ class _CollectKeywordNameToKeywordFound:
 def collect_keyword_name_to_keyword_found(
     completion_context: ICompletionContext,
 ) -> Dict[str, List[IKeywordFound]]:
-    completion_context.memo.clear()
     collector = _CollectKeywordNameToKeywordFound()
-    _collect_following_imports(completion_context, collector)
+    _collect_from_context(completion_context, collector)
     return collector.keyword_name_to_keyword_found
 
 
@@ -544,5 +531,4 @@ def collect_keywords(
     """
     Collects all the keywords that are available to the given completion_context.
     """
-    completion_context.memo.clear()
-    _collect_following_imports(completion_context, collector)
+    _collect_from_context(completion_context, collector)
