@@ -1,4 +1,3 @@
-import re
 from robotframework_ls.impl.protocols import (
     ICompletionContext,
     IDefinition,
@@ -7,15 +6,16 @@ from robotframework_ls.impl.protocols import (
     IKeywordCollector,
     IVariablesCollector,
     IKeywordFound,
-    ILibraryDoc,
+    AbstractKeywordCollector,
+    IRobotToken,
+    IVariableFound,
+    AbstractVariablesCollector,
 )
-from robocorp_ls_core.protocols import check_implements
-from typing import Optional, Sequence
-from robocorp_ls_core.lsp import RangeTypedDict
+from robocorp_ls_core.protocols import check_implements, IDocumentSelection
+from typing import Optional, Sequence, List
+from robocorp_ls_core.lsp import RangeTypedDict, MarkupContentTypedDict, MarkupKind
 from robocorp_ls_core.basic import isinstance_name
 
-_RF_VARIABLE = re.compile(r"([$|&|@]{[\w\s]+})")
-
 
 class _DefinitionFromKeyword(object):
     def __init__(self, keyword_found: IKeywordFound):
@@ -33,6 +33,12 @@ class _DefinitionFromKeyword(object):
         self.scope_col_offset: Optional[int] = keyword_found.scope_col_offset
         self.scope_end_col_offset: Optional[int] = keyword_found.scope_end_col_offset
 
+    def hover_docs(self) -> MarkupContentTypedDict:
+        from robotframework_ls.html_to_markdown import escape
+
+        keyword_name = escape(self.keyword_name)
+        return {"kind": MarkupKind.Markdown, "value": f"Keyword: **{keyword_name}**"}
+
     def __str__(self):
         return "DefinitionFromKeyword[%s, %s:%s]" % (
             self.keyword_name,
@@ -65,6 +71,18 @@ class _DefinitionFromLibrary(object):
         self.scope_col_offset: Optional[int] = None
         self.scope_end_col_offset: Optional[int] = None
 
+    def hover_docs(self) -> MarkupContentTypedDict:
+        from robotframework_ls.html_to_markdown import escape
+        from robotframework_ls.impl.robot_specbuilder import docs_and_format
+
+        library_name = escape(self.library_doc.name)
+        docs, doc_format = docs_and_format(self.library_doc)
+
+        return {
+            "kind": doc_format,
+            "value": f"Library: **{library_name}**\n\n{docs}",
+        }
+
     def __str__(self):
         return "DefinitionFromLibrary[%s]" % (self.source,)
 
@@ -95,6 +113,23 @@ class _DefinitionFromResource(object):
         self.scope_col_offset: Optional[int] = None
         self.scope_end_col_offset: Optional[int] = None
 
+    def hover_docs(self) -> MarkupContentTypedDict:
+        from robotframework_ls.impl import ast_utils
+        from robotframework_ls.html_to_markdown import escape
+        import os
+
+        ast = self.resource_doc.get_ast()
+        documentation = ast_utils.get_documentation_as_markdown(ast)
+        if documentation:
+            documentation = "\n\n" + documentation
+
+        name = escape(os.path.splitext(os.path.basename(self.source))[0])
+
+        return {
+            "kind": MarkupKind.Markdown,
+            "value": f"Resource: {name}{documentation}",
+        }
+
     def __str__(self):
         return "DefinitionFromResource[%s]" % (self.source,)
 
@@ -125,6 +160,14 @@ class _DefinitionFromVariableImport(object):
         self.scope_col_offset: Optional[int] = None
         self.scope_end_col_offset: Optional[int] = None
 
+    def hover_docs(self) -> MarkupContentTypedDict:
+        from robotframework_ls.html_to_markdown import escape
+        import os
+
+        name = escape(os.path.splitext(os.path.basename(self.source))[0])
+
+        return {"kind": MarkupKind.Markdown, "value": f"Variable Import: {name}"}
+
     def __str__(self):
         return "DefinitionFromVariableImport[%s]" % (self.source,)
 
@@ -153,6 +196,34 @@ class _DefinitionFromVariable(object):
         self.scope_col_offset: Optional[int] = None
         self.scope_end_col_offset: Optional[int] = None
 
+    def hover_docs(self) -> MarkupContentTypedDict:
+        from robotframework_ls.html_to_markdown import escape
+
+        variable_found = self.variable_found
+        variable_name = "".join(
+            (
+                variable_found.variable_kind,
+                ": ",
+                "**",
+                escape(variable_found.variable_name.strip()),
+                "**",
+            )
+        )
+
+        variable_value = ""
+        if variable_found.variable_value:
+            variable_value = "".join(
+                (
+                    "\n\nValue: ",
+                    escape(variable_found.variable_value.strip()),
+                )
+            )
+
+        return {
+            "kind": MarkupKind.Markdown,
+            "value": (f"{variable_name}{variable_value}"),
+        }
+
     def __str__(self):
         return "_DefinitionFromVariable(%s[%s, %s:%s])" % (
             self.variable_found.__class__.__name__,
@@ -167,7 +238,7 @@ class _DefinitionFromVariable(object):
         _: IDefinition = check_implements(self)
 
 
-class _FindDefinitionKeywordCollector(object):
+class _FindDefinitionKeywordCollector(AbstractKeywordCollector):
     def __init__(self, match_name):
         from robotframework_ls.impl.string_matcher import RobotStringMatcher
         from robotframework_ls.impl.string_matcher import (
@@ -195,52 +266,23 @@ class _FindDefinitionKeywordCollector(object):
                 self.matches.append(definition)
                 return
 
-    def on_resolved_library(
-        self,
-        completion_context: ICompletionContext,
-        library_node,
-        library_doc: ILibraryDoc,
-    ):
-        pass
-
-    def on_unresolved_library(
-        self,
-        completion_context: ICompletionContext,
-        library_name: str,
-        lineno: int,
-        end_lineno: int,
-        col_offset: int,
-        end_col_offset: int,
-        error_msg: Optional[str],
-    ):
-        pass
-
-    def on_unresolved_resource(
-        self,
-        completion_context: ICompletionContext,
-        resource_name: str,
-        lineno: int,
-        end_lineno: int,
-        col_offset: int,
-        end_col_offset: int,
-    ):
-        pass
-
     def __typecheckself__(self) -> None:
         _: IKeywordCollector = check_implements(self)
 
 
-class _FindDefinitionVariablesCollector(object):
-    def __init__(self, sel, token, robot_string_matcher):
-        self.matches = []
+class _FindDefinitionVariablesCollector(AbstractVariablesCollector):
+    def __init__(
+        self, sel: IDocumentSelection, token: IRobotToken, robot_string_matcher
+    ):
+        self.matches: List[IDefinition] = []
         self.sel = sel
         self.token = token
         self.matcher = robot_string_matcher
 
-    def accepts(self, variable_name):
-        return self.matcher.is_same_robot_name(variable_name)
+    def accepts(self, variable_name: str) -> bool:
+        return self.matcher.is_same_variable_name(variable_name)
 
-    def on_variable(self, variable_found):
+    def on_variable(self, variable_found: IVariableFound):
         definition = _DefinitionFromVariable(variable_found)
         self.matches.append(definition)
 
@@ -260,7 +302,9 @@ def find_keyword_definition(
     from robotframework_ls.impl.collect_keywords import collect_keywords
     from robotframework_ls.impl import ast_utils
 
-    token = ast_utils.get_keyword_name_token(token_info.node, token_info.token)
+    token = ast_utils.get_keyword_name_token(
+        token_info.stack, token_info.node, token_info.token
+    )
     if token is None:
         if token_info.token.type == token_info.token.KEYWORD_NAME:
             if isinstance_name(token_info.node, "KeywordName"):
@@ -362,13 +406,12 @@ def find_definition_extended(
                     ast_utils.create_range_from_token(token),
                 )
 
-    token_info = completion_context.get_current_variable()
-    if token_info is not None:
-        token = token_info.token
+    var_token_info = completion_context.get_current_variable()
+    if var_token_info is not None:
+        token = var_token_info.token
         value = token.value
-        match = next(iter(_RF_VARIABLE.findall(value)), value)
         collector = _FindDefinitionVariablesCollector(
-            completion_context.sel, token, RobotStringMatcher(match)
+            completion_context.sel, token, RobotStringMatcher(value)
         )
         collect_variables(completion_context, collector)
         return _DefinitionInfo(
