@@ -9,6 +9,7 @@ OPTION_ROBOT_LANGUAGE_SERVER_TCP_PORT = "robot.language-server.tcp-port"
 OPTION_ROBOT_LANGUAGE_SERVER_ARGS = "robot.language-server.args"
 
 OPTION_ROBOT_VARIABLES = "robot.variables"
+OPTION_ROBOT_VARIABLES_LOAD_FROM_ARGUMENTS_FILE = "robot.loadVariablesFromArgumentsFile"
 
 OPTION_ROBOT_LINT_ROBOCOP_ENABLED = "robot.lint.robocop.enabled"
 
@@ -16,8 +17,13 @@ OPTION_ROBOT_LINT_ENABLED = "robot.lint.enabled"
 OPTION_ROBOT_LINT_UNDEFINED_KEYWORDS = "robot.lint.undefinedKeywords"
 OPTION_ROBOT_LINT_UNDEFINED_LIBRARIES = "robot.lint.undefinedLibraries"
 OPTION_ROBOT_LINT_UNDEFINED_RESOURCES = "robot.lint.undefinedResources"
+OPTION_ROBOT_LINT_UNDEFINED_VARIABLE_IMPORTS = "robot.lint.undefinedVariableImports"
 OPTION_ROBOT_LINT_KEYWORD_CALL_ARGUMENTS = "robot.lint.keywordCallArguments"
 
+OPTION_ROBOT_LINT_VARIABLES = "robot.lint.variables"
+OPTION_ROBOT_LINT_IGNORE_VARIABLES = "robot.lint.ignoreVariables"
+
+
 OPTION_ROBOT_CODE_FORMATTER = "robot.codeFormatter"
 
 OPTION_ROBOT_CODE_FORMATTER_ROBOTIDY = "robotidy"
@@ -43,6 +49,8 @@ OPTION_ROBOT_COMPLETION_SECTION_HEADERS_FORM_PLURAL = "plural"
 OPTION_ROBOT_COMPLETION_SECTION_HEADERS_FORM_SINGULAR = "singular"
 OPTION_ROBOT_COMPLETION_SECTION_HEADERS_FORM_BOTH = "both"
 
+OPTION_ROBOT_SHOW_CODE_LENSES = "robot.codeLens.enable"
+
 # Options which must be set as environment variables.
 ENV_OPTION_ROBOT_DAP_TIMEOUT = "ROBOT_DAP_TIMEOUT"
 
