@@ -1,7 +1,7 @@
 from robocorp_ls_core.python_ls import PythonLanguageServer
 from robocorp_ls_core.basic import overrides
 from robocorp_ls_core.robotframework_log import get_logger
-from typing import Optional, List, Dict, Deque
+from typing import Optional, List, Dict, Deque, Tuple, Sequence
 from robocorp_ls_core.protocols import IConfig, IMonitor, ITestInfoTypedDict, IWorkspace
 from functools import partial
 from robocorp_ls_core.jsonrpc.endpoint import require_monitor
@@ -23,6 +23,7 @@ from robotframework_ls.impl.protocols import (
     IDefinition,
     ICompletionContext,
     EvaluatableExpressionTypedDict,
+    IVariablesFromArgumentsFileLoader,
 )
 from robocorp_ls_core.watchdog_wrapper import IFSObserver
 import itertools
@@ -34,6 +35,55 @@ import threading
 log = get_logger(__name__)
 
 
+def complete_all(
+    completion_context: ICompletionContext,
+) -> List[CompletionItemTypedDict]:
+    from robotframework_ls.impl import section_name_completions
+    from robotframework_ls.impl import keyword_completions
+    from robotframework_ls.impl import variable_completions
+    from robotframework_ls.impl import dictionary_completions
+    from robotframework_ls.impl import filesystem_section_completions
+    from robotframework_ls.impl import keyword_parameter_completions
+    from robotframework_ls.impl import auto_import_completions
+    from robotframework_ls.impl.collect_keywords import (
+        collect_keyword_name_to_keyword_found,
+    )
+    from robotframework_ls.impl import ast_utils
+
+    ret = section_name_completions.complete(completion_context)
+    if not ret:
+        ret.extend(filesystem_section_completions.complete(completion_context))
+
+    if not ret:
+        token_info = completion_context.get_current_token()
+        if token_info is not None:
+            token = ast_utils.get_keyword_name_token(
+                token_info.stack, token_info.node, token_info.token
+            )
+            if token is not None:
+                keyword_name_to_keyword_found: Dict[
+                    str, List[IKeywordFound]
+                ] = collect_keyword_name_to_keyword_found(completion_context)
+                ret.extend(keyword_completions.complete(completion_context))
+                ret.extend(
+                    auto_import_completions.complete(
+                        completion_context, keyword_name_to_keyword_found
+                    )
+                )
+                return ret
+
+    if not ret:
+        ret.extend(variable_completions.complete(completion_context))
+
+    if not ret:
+        ret.extend(dictionary_completions.complete(completion_context))
+
+    if not ret:
+        ret.extend(keyword_parameter_completions.complete(completion_context))
+
+    return ret
+
+
 class RobotFrameworkServerApi(PythonLanguageServer):
     """
     This is a custom server. It uses the same message-format used in the language
@@ -69,12 +119,16 @@ class RobotFrameworkServerApi(PythonLanguageServer):
                 raise
 
         self.libspec_manager = libspec_manager
-        self._version = None
+        self._version: Optional[str] = None
+        self._logged_version_error = False
         self._next_time = partial(next, itertools.count(0))
         from collections import deque
 
         self._completion_contexts_saved_lock = threading.Lock()
         self._completion_contexts_saved: Deque[ICompletionContext] = deque()
+        self._variables_from_arguments_files_loader: Sequence[
+            IVariablesFromArgumentsFileLoader
+        ] = []
 
     @overrides(PythonLanguageServer._create_config)
     def _create_config(self) -> IConfig:
@@ -82,7 +136,14 @@ class RobotFrameworkServerApi(PythonLanguageServer):
 
         return RobotConfig()
 
-    def m_version(self):
+    def m_version(self) -> str:
+        """
+        Kind of a code-smell:
+
+        Either a string with the version is provided or an error message.
+
+        It's now part of the API, so, keep as is.
+        """
         if self._version is not None:
             return self._version
         try:
@@ -114,17 +175,69 @@ class RobotFrameworkServerApi(PythonLanguageServer):
         self._version = version
         return self._version
 
-    def _check_min_version(self, min_version):
+    def _compute_min_version_error(self, min_version: Tuple[int, int]) -> Optional[str]:
         from robocorp_ls_core.basic import check_min_version
 
-        version = self.m_version()
-        return check_min_version(version, min_version)
+        v_or_error = self.m_version()
+        try:
+            # Check if it's an error or a valid version.
+            tuple(int(x) for x in v_or_error.split("."))
+        except:
+            return v_or_error  # This is an error message
+
+        if not check_min_version(v_or_error, min_version):
+            return f"Expected Robot Framework version: {'.'.join((str(x) for x in min_version))}. Found: {v_or_error}"
+        return None
+
+    def _check_and_log_rf_dependency_version(self):
+        error = self._compute_min_version_error((3, 2))
+        if error is not None:
+            if self._logged_version_error:
+                log.info(
+                    "Problem with `robotframework` dependency. See previous errors."
+                )
+            else:
+                log.info(error)
+                self._logged_version_error = True
+            return False
+        return True
 
     @overrides(PythonLanguageServer.m_workspace__did_change_configuration)
     def m_workspace__did_change_configuration(self, **kwargs):
+        from robotframework_ls.impl.robot_lsp_constants import (
+            OPTION_ROBOT_VARIABLES_LOAD_FROM_ARGUMENTS_FILE,
+        )
+
         PythonLanguageServer.m_workspace__did_change_configuration(self, **kwargs)
         self.libspec_manager.config = self.config
 
+        try:
+            variables_from_arguments_files = self.config.get_setting(
+                OPTION_ROBOT_VARIABLES_LOAD_FROM_ARGUMENTS_FILE, str, ""
+            ).strip()
+            if not variables_from_arguments_files:
+                self._variables_from_arguments_files_loader = []
+
+            else:
+                from robotframework_ls.impl.variables_from_arguments_file import (
+                    VariablesFromArgumentsFileLoader,
+                )
+
+                created_variables_from_arguments_files = []
+
+                for v in variables_from_arguments_files.split(","):
+                    v = v.strip()
+                    created_variables_from_arguments_files.append(
+                        VariablesFromArgumentsFileLoader(v)
+                    )
+                self._variables_from_arguments_files_loader = (
+                    created_variables_from_arguments_files
+                )
+        except:
+            log.exception(
+                f"Error getting options: {OPTION_ROBOT_VARIABLES_LOAD_FROM_ARGUMENTS_FILE}"
+            )
+
     @overrides(PythonLanguageServer.lint)
     def lint(self, *args, **kwargs):
         pass  # No-op for this server.
@@ -156,12 +269,11 @@ class RobotFrameworkServerApi(PythonLanguageServer):
         return robot_workspace
 
     def m_lint(self, doc_uri):
-        if not self._check_min_version((3, 2)):
+        error = self._compute_min_version_error((3, 2))
+        if error is not None:
             from robocorp_ls_core.lsp import Error
 
-            msg = self.m_version()
-            log.info(msg)
-            return [Error(msg, (0, 0), (1, 0)).to_lsp_diagnostic()]
+            return [Error(error, (0, 0), (1, 0)).to_lsp_diagnostic()]
 
         func = partial(self._threaded_lint, doc_uri)
         func = require_monitor(func)
@@ -300,56 +412,14 @@ class RobotFrameworkServerApi(PythonLanguageServer):
         return completions
 
     def _complete_from_completion_context(self, completion_context):
-        from robotframework_ls.impl import section_name_completions
-        from robotframework_ls.impl import keyword_completions
-        from robotframework_ls.impl import variable_completions
-        from robotframework_ls.impl import dictionary_completions
-        from robotframework_ls.impl import filesystem_section_completions
-        from robotframework_ls.impl import keyword_parameter_completions
-        from robotframework_ls.impl import auto_import_completions
-        from robotframework_ls.impl.collect_keywords import (
-            collect_keyword_name_to_keyword_found,
-        )
-        from robotframework_ls.impl import ast_utils
-
         with self._completion_contexts_saved_lock:
-            # Keep up to 3 saved contexts there.
+            # Keep up to 3 saved contexts there (used to resolve
+            # completion items afterwards).
             while len(self._completion_contexts_saved) > 2:
                 self._completion_contexts_saved.popleft()
             self._completion_contexts_saved.append(completion_context)
 
-        ret = section_name_completions.complete(completion_context)
-        if not ret:
-            ret.extend(filesystem_section_completions.complete(completion_context))
-
-        if not ret:
-            token_info = completion_context.get_current_token()
-            if token_info is not None:
-                token = ast_utils.get_keyword_name_token(
-                    token_info.node, token_info.token
-                )
-                if token is not None:
-                    keyword_name_to_keyword_found: Dict[
-                        str, List[IKeywordFound]
-                    ] = collect_keyword_name_to_keyword_found(completion_context)
-                    ret.extend(keyword_completions.complete(completion_context))
-                    ret.extend(
-                        auto_import_completions.complete(
-                            completion_context, keyword_name_to_keyword_found
-                        )
-                    )
-                    return ret
-
-        if not ret:
-            ret.extend(variable_completions.complete(completion_context))
-
-        if not ret:
-            ret.extend(dictionary_completions.complete(completion_context))
-
-        if not ret:
-            ret.extend(keyword_parameter_completions.complete(completion_context))
-
-        return ret
+        return complete_all(completion_context)
 
     def m_section_name_complete(self, doc_uri, line, col):
         from robotframework_ls.impl import section_name_completions
@@ -482,6 +552,9 @@ class RobotFrameworkServerApi(PythonLanguageServer):
         if not text:
             return []
 
+        if not self._check_and_log_rf_dependency_version():
+            return []
+
         if options is None:
             options = {}
         tab_size = options.get("tabSize", 4)
@@ -515,9 +588,10 @@ class RobotFrameworkServerApi(PythonLanguageServer):
             new_contents = robot_source_format(text, space_count=tab_size)
 
         else:
-            if not self._check_min_version((4, 0)):
+            error = self._compute_min_version_error((4, 0))
+            if error is not None:
                 log.critical(
-                    f"To use the robotidy formatter, at least Robot Framework 4 is needed. Found: {self.m_version()}"
+                    f"To use the robotidy formatter, at least Robot Framework 4 is needed. {error}"
                 )
                 return []
 
@@ -553,8 +627,7 @@ class RobotFrameworkServerApi(PythonLanguageServer):
     ):
         from robotframework_ls.impl.completion_context import CompletionContext
 
-        if not self._check_min_version((3, 2)):
-            log.info("robotframework version too old.")
+        if not self._check_and_log_rf_dependency_version():
             return None
         workspace = self.workspace
         if not workspace:
@@ -572,6 +645,7 @@ class RobotFrameworkServerApi(PythonLanguageServer):
             workspace=workspace,
             config=self.config,
             monitor=monitor,
+            variables_from_arguments_files_loader=self._variables_from_arguments_files_loader,
         )
 
     def m_signature_help(self, doc_uri: str, line: int, col: int):
@@ -895,13 +969,14 @@ class RobotFrameworkServerApi(PythonLanguageServer):
         }
 
     def m_semantic_tokens_from_code_full(
-        self, prefix: str = "", full_code: str = "", indent: str = ""
+        self, prefix: str = "", full_code: str = "", indent: str = "", uri: str = ""
     ):
         func = partial(
             self.threaded_semantic_tokens_from_code_full,
             prefix=prefix,
             full_code=full_code,
             indent=indent,
+            uri=uri,
         )
         func = require_monitor(func)
         return func
@@ -911,17 +986,24 @@ class RobotFrameworkServerApi(PythonLanguageServer):
         prefix: str,
         full_code: str,
         indent: str,
+        uri: str,
         monitor: Optional[IMonitor] = None,
     ):
-        from robotframework_ls.impl.semantic_tokens import semantic_tokens_full_from_ast
+        from robotframework_ls.impl.semantic_tokens import semantic_tokens_full
+        from robotframework_ls.impl.completion_context import CompletionContext
 
         try:
             from robotframework_ls.impl.robot_workspace import RobotDocument
 
-            doc = RobotDocument("")
-            doc.source = full_code
-            ast = doc.get_ast()
-            data = semantic_tokens_full_from_ast(ast, monitor)
+            document = RobotDocument(uri, full_code)
+            completion_context = CompletionContext(
+                document,
+                config=self.config,
+                monitor=monitor,
+                workspace=self.workspace,
+            )
+
+            data = semantic_tokens_full(completion_context)
             if not prefix:
                 return {"resultId": None, "data": data}
 
